<?php


namespace App\Services\goods;


use Api\PhpServices\Idgen\Idgen;
use Api\PhpServices\Ksy\Ks3Api;
use Api\PhpServices\Ksy\Ksyun;
use Api\PhpServices\Sensitive\Sensitive;
use Api\PhpUtils\Common\BaseConvert;
use Api\PhpUtils\Common\GoodsSkuId;
use Api\PhpUtils\Http\HttpUtil;
use Api\PhpUtils\Lock\FrequencyLockUtil;
use Api\PhpUtils\Log\FileLog;
use App\Exception\custom\GoodsException;
use App\Models\goods\mysql\Category;
use App\Models\goods\mysql\GoodsOperationRecord;
use App\Models\goods\mysql\GoodsRefundRecord;
use App\Models\goods\mysql\GoodsSku;
use App\Models\goods\mysql\GoodsSkuPicRecord;
use App\Models\goods\mysql\GoodsSkuSubShop;
use App\Models\goods\mysql\GoodsSnapshot;
use App\Models\goods\mysql\GoodsSpu;
use App\Models\goods\mysql\LifeAccountShopNum;
use App\Models\goods\mysql\Ota;
use App\Models\goods\mysql\PaySuccessGoodsCallbackRecord;
use App\Models\goods\mysql\ErshouGoodsSku;
use App\Models\goods\mysql\PindanGoodsSnapshot;
use App\Models\goods\mysql\Shop;
use App\Models\marketing\mysql\Distributor;
use App\Models\marketing\mysql\Marketing;
use App\Models\marketing\mysql\MarketingGoods;
use App\Models\shop\mysql\SubShop;
use App\Services\marketing\DistributorService;
use App\Services\marketing\MarketingGoodsService;
use App\Services\marketing\MarketingService;
use App\Services\shop\ShopService;
use App\Services\common\CommonService;

use Api\PhpUtils\Http\Request;
use App\Exception\custom\InterfaceException;

class ErshouGoodsService
{
    /**
     * 拼单商品
     * @param $arr
     * @return string
     * @throws GoodsException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    public static function ershouAddGoods($arr)
    {
        GoodsSpu::beginTransaction();

        $skuParams = json_decode($arr["sku_json"], true);
        if (!empty($skuParams)) {
            foreach ($skuParams as $params) {
                $userId = $params["publish_user_id"];
            }
            list($lifeAccountId, $merchantId) = self::getLifeAccountInfo($userId);
            $lifeAccount = [
                "life_account_id" => $lifeAccountId,
                "merchant_id" => $merchantId,
            ];
            foreach ($skuParams as $params) {
                //自定义的商品名
                $params["life_account_id"] = $lifeAccountId;
                $params["goods_name"] = $lifeAccountId . "发布的二手商品";
                $shopId = GoodsService::addShop($lifeAccount);
                $spuData = self::addGoodsSpu($shopId, $params);
                $skuId = self::addGoodsSku($spuData, $shopId, $params);
                //商品操作记录
                $record = [
                    "goods_spu_id" => $spuData["goods_spu_id"],
                    "goods_sku_id" => $skuId,
                    "operator_id" => empty($params["publish_user_id"]) ? 0 : $params["publish_user_id"],
                    "operator_name" => $params["user_name"],
                    "goods_status" => GoodsSku::STATUS_PASS,
                    "online_status" => GoodsSku::ONLINE_STATUS_ONLINE,
                    "before_version" => 1,
                    "after_version" => 1,
                    "note" => empty($params["rejected_reason"]) ? "" : $params["rejected_reason"],
                ];
                GoodsOperationRecord::insertRecord($record);
            }
        }
        if (!GoodsSpu::commit()) {
            GoodsSpu::rollback();
            throw new GoodsException(["cus" => 0]);
        }
        return true;
    }

    /**
     * 获取生活号信息，没有的话默认创建
     * @param $userId
     * @return array
     * @throws GoodsException
     * @throws InterfaceException
     */
    private static function getLifeAccountInfo($userId)
    {
        $url = config('interface', 'merchant.lifeaccount.get_lid_by_uid');
        if (empty($url)) {
            throw new InterfaceException(['cus' => 0]);
        }
        $res = HttpUtil::get($url, ["uid" => $userId]);
        $accountRes = empty($res["response"]["result"]) ? [] : $res["response"]["result"][0];
        if (!empty($accountRes)) {
            $lifeAccountId = $accountRes["life_account_id"];
            $merchantId = $accountRes["merchant_id"];
        } else {
            $url = config('interface', 'merchant.life_account.create_unauthorized');
            if (empty($url)) {
                throw new InterfaceException(['cus' => 0]);
            }
            $lifeAccountInfo = HttpUtil::get($url, ["user_id" => $userId]);
            $lifeAccountId = empty($lifeAccountInfo["life_account_id"]) ? "" : $lifeAccountInfo["life_account_id"];
            $merchantId = empty($lifeAccountInfo["merchant_id"]) ? "" : $lifeAccountInfo["merchant_id"];
            if (empty($lifeAccountId) || empty($merchantId)) {
                throw new GoodsException(['cus' => 0]);
            }
        }

        return [$lifeAccountId, $merchantId];
    }

    /**
     * 添加spu
     * @param $shopId
     * @param array $spuData
     * @return mixed
     * @throws GoodsException
     */
    private static function addGoodsSpu($shopId, $spuData = [])
    {
        $spu = GoodsSpu::get("*", ["goods_name" => $spuData["goods_name"], "life_account_id" => $spuData["life_account_id"]]);
        if (!empty($spu)) {
            return $spu;
        }

        $res = GoodsService::getIdgenId(substr($shopId, -2), "goods");
        if (empty($res)) {
            throw new GoodsException(['cus' => 2]);
        }
        $spuId = $res[0];

        $spuParams = [
            "goods_spu_id" => $spuId,
            "shop_id" => $shopId,
            "life_account_id" => $spuData["life_account_id"],
            "publish_user_id" => $spuData["publish_user_id"],
            "goods_name" => $spuData["goods_name"],
            "desc_pic_url" => $spuData["desc_pic_url"],
            "desc" => $spuData["desc"],
            "rule_limit" => empty($skuData["rule_limit"]) ? 0 : $skuData["rule_limit"],
            "goods_type" => empty($spuData["goods_type"]) ? GoodsSpu::TYPE_ENTITY : $spuData["goods_type"],
            "goods_sub_type" => empty($spuData["goods_sub_type"]) ? GoodsSpu::SUB_TYPE_ERSHOU : $spuData["goods_sub_type"],
            "belong_type" => empty($spuData["belong_type"]) ? GoodsSpu::BELONG_TYPE_DISANFANG : $spuData["belong_type"],
            "goods_sale_type" => empty($spuData["setmeal"]) ? GoodsSpu::SALE_TYPE_NO_COMBINATION : GoodsSpu::SALE_TYPE_COMBINATION,
            "can_show_es" => GoodsSpu::CAN_SHOW_ES_NO,
        ];
        GoodsSpu::save($spuParams);
        return $spuParams;
    }

    /**
     * 添加拼单活动sku
     * @param $spuData
     * @param $shopId
     * @param array $skuData
     * @return string
     * @throws GoodsException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    private static function addGoodsSku($spuData, $shopId, $skuData = [])
    {
        $res = GoodsService::getIdgenId(substr($shopId, -2), "goods");
        if (empty($res)) {
            throw new GoodsException(['cus' => 2]);
        }
        $skuId = GoodsSkuId::generateGoodSkuId($res[0], "00", "000", $spuData["goods_type"], $spuData["belong_type"], "002");
        $checkGoodsDesc = GoodsService::isHaveSensitive($skuData["desc"], 2);
        if ($checkGoodsDesc) {
            throw new GoodsException(['cus' => 20]);
        }

        $skuParams = [
            "goods_sku_id" => $skuId,
            "goods_spu_id" => $spuData["goods_spu_id"],
            "life_account_id" => $skuData["life_account_id"],
            "publish_user_id" => empty($skuData["publish_user_id"]) ? "" : $skuData["publish_user_id"],
            "category_1_id" => 0,
            "category_2_id" => 0,
            "goods_name" => $skuData["goods_name"],
            "desc_pic_url" => $skuData["desc_pic_url"],
            "desc" => $skuData["desc"],
            "original_price" => empty($skuData["price"]) ? "" : $skuData["price"] * 100,
            "price" => empty($skuData["price"]) ? "" : $skuData["price"] * 100,
            "audit_status" => ErshouGoodsSku::STATUS_AUDIT,
            "online_status" => ErshouGoodsSku::ONLINE_STATUS_NO_ONLINE,
            "location_name" => empty($skuData["location_name"]) ? "" : $skuData["location_name"],
            "longitude" => empty($skuData["longitude"]) ? "" : $skuData["longitude"],
            "latitude" => empty($skuData["latitude"]) ? "" : $skuData["latitude"],
            "address" => empty($skuData["address"]) ? "" : $skuData["address"],
            "province" => empty($skuData["province"]) ? "" : $skuData["province"],
            "city" => empty($skuData["city"]) ? "" : $skuData["city"],
            "area" => empty($skuData["area"]) ? "" : $skuData["area"],
        ];

        $res = ErshouGoodsSku::save($skuParams);
        if ($res === false) {
            GoodsSpu::rollback();
            throw new GoodsException(["cus" => 0]);
        }
        GoodsService::addGoodsSkuPicRecord($skuId, $skuData);
        return $skuId;
    }
}