<?php


namespace App\Services\user;

use Api\PhpUtils\Mon\MonUtil;
use App\Models\user\mysql\UserWechatBind;
use Api\PhpUtils\Http\Request;
use App\Services\user\Weixin\WxBizDataCrypt;;
use App\Exception\BaseException;
use App\Exception\custom\UserException;
use App\Exception\custom\CodeSpecialException;
use Api\PhpUtils\Validate\Validate;
use Api\PhpUtils\Redis\RedisUtil;
use Api\PhpServices\JwUser\JwUser;
use App\Services\common\CommonService;
use Api\PhpUtils\Log\FileLog;
use Api\PhpUtils\Lock\FrequencyLockUtil;

class UserService
{
    private static $code = '';
    private static $grantType = 'authorization_code';
    private static $loginUrl = 'https://api.weixin.qq.com/sns/jscode2session';
    private static $wechatAppId = '';
    private static $secret = "";
    private static $redis = "";
    private static $keyCode = "";
    private static $openid = "";

    /**
     * 微信静默登陆
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase|mixed
     * @throws UserException
     */
    public static function wechatLogin($params)
    {
        self::$code = !empty($params['code']) ? $params['code'] : '';//小程序授权code
 
        $appid = !empty($params['appid']) ? $params['appid'] : ''; //登陆来源
        $inviteUserId = !empty($params['invite_user_id']) ? intval($params['invite_user_id']) : 0;//邀请用户id
        $activityType = !empty($params['activity_type']) ? intval($params['activity_type']) : 0;//活动类型
        //验证登陆参数
        self::checkWechatLoginParams($params);
        $lock = self::getWechatLoginLock($params);
        if(!empty($lock)){
           return $lock;
        }
        //设置小程序配置
        self::setWechatConfig($appid);
        //获取小程序用户信息
        $response = self::getJscode2session();
        //检查系统是已注册
        $openid = !empty($response['openid']) ? $response['openid'] : '';
        $user = UserWechatBind::getRecordMaster(['openid' => $openid]);
        //如果系统不存在，用户信息，则注册新用户
        if (empty($user) && !empty($response)) {
            $insert = [
                'openid' => !empty($response['openid']) ? $response['openid'] : '',
                'union_id' => !empty($response['unionid']) ? $response['unionid'] : '',
                'session_key' => !empty($response['session_key']) ? $response['session_key'] : '', //默认7200有效期
                'invite_user_id' => $inviteUserId,
                'activity_type' => $activityType,
                'create_time' => date("Y-m-d H:i:s"),
                'appid' => $appid,
                'wechat_app_id' => self::$wechatAppId,
            ];

            $sessionKey = !empty($response['session_key']) ? $response['session_key'] : '';
            //$newWechatId = UserWechatBind::save($insert);
            $newWechatId = UserWechatBind::insertDuplicate($insert, ['session_key' => $sessionKey, 'create_time' => date("Y-m-d H:i:s")]);
            if (!$newWechatId) {
                FileLog::error("wechatLogin：登陆创建用户失败", json_encode($params, JSON_UNESCAPED_UNICODE), '', 'jianghaiming@yidian-inc.com');
                FileLog::error("wechatLogin：登陆创建用户失败--insert", json_encode($insert, JSON_UNESCAPED_UNICODE), '', 'jianghaiming@yidian-inc.com');
                throw new UserException(['cus' => 2]);
            }

            $user['wechat_id'] = $newWechatId;
            $user['openid'] = $response['openid'];
            $user['session_key'] = $response['session_key'];
        } else {
            if (empty($user)) {
                FileLog::error("wechatLogin：登陆创建用户失败-- 微信response", json_encode($response, JSON_UNESCAPED_UNICODE), '', 'jianghaiming@yidian-inc.com');
                throw new UserException(['cus' => 2]);
            }

            if (empty($response)) {
                FileLog::error("wechatLogin：登陆创建用户失败-- 微信response 获取失败", json_encode($params, JSON_UNESCAPED_UNICODE), '', 'jianghaiming@yidian-inc.com');
                throw new UserException(['cus' => 2]);
            }
            $insert = [
                'session_key' => !empty($response['session_key']) ? $response['session_key'] : '',
            ];
            $newWechatId = UserWechatBind::save($insert, ['openid' => $openid]);
        }
        $user['third_session'] = self::generate3rdSession($openid);

        self::$redis->set(self::$keyCode, json_encode($user),120);
        return $user;
    }

    /**
     * 验证登陆参数
     * @param $params
     * @throws UserException
     */
    public static function checkWechatLoginParams($params)
    {
        if (empty(self::$code)) {
            FileLog::error("wechatLogin:获取参数为空", json_encode($params, JSON_UNESCAPED_UNICODE), '', 'jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 0]);
        }
        if (empty($params['appid'])) {
            throw new UserException(['cus' => UserException::APP_ID_NULL]);
        }
    }

    /**
     * 初始化微信配置
     * @param $wechatFrom
     */
    public static function setWechatConfig($wechatFrom)
    {
        //需要授权微信
        self::$wechatAppId = \Yaf\Registry::get('config')->wechat->appid->$wechatFrom;
        self::$secret = \Yaf\Registry::get('config')->wechat->secret->$wechatFrom;
    }

    /**
     * 重复提交
     * @param $code
     * @return bool|mixed
     */
    public static function getWechatLoginLock($params){
        self::$redis = RedisUtil::getInstance('cache');
        $key = "wx_wechat_login_".self::$code;
        self::$keyCode = "wx_wechat_login_code_".self::$code;
        $lock = FrequencyLockUtil::isLocked($key,1,120);
        if ($lock) {
            sleep(1);
            $user = self::$redis->get(self::$keyCode);
            if (!empty($user)) {
                $user = json_decode($user,true);
            }
            FileLog::error("wechatLogin: lock", json_encode($user, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            FileLog::error("wechatLogin: lock--params", json_encode($params, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            return $user;
        }
        return false;
    }

    /**
     * 获取微信用户信息 openid
     * @param $code
     * @return array|mixed
     * @throws UserException
     */
    public static function getJscode2session(){
        //请求接口，获取用户openid
        $params = [
            'appid' => self::$wechatAppId,
            'secret' => self::$secret,
            'js_code' => self::$code,
            'grant_type' => self::$grantType,
        ];
        $begin = microtime(true);
        $response = (new Request())->post(self::$loginUrl, $params);

        $end = microtime(true);
        $totalTime = round(($end - $begin), 4) * 1000;
        if ($response['code'] == 0) {
            $response = $response['response'];
        }
        //mon打点
        $env = \Yaf\Application::app()->environ() ?? "dev";
        MonUtil::proxyMon(self::$loginUrl, $response['code'], 'weixin_'.$env, $totalTime);
        if (empty($response) || array_key_exists('errcode', $response)) {
            //throw new \Exception('获取openid失败:' . $response['errcode']);
            FileLog::error("wechatLogin：获取openid失败", json_encode($params, JSON_UNESCAPED_UNICODE), '', 'jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 1]);
        }
        return $response;
    }

    /**
     * 绑定手机号
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws UserException
     * @throws \App\Exception\custom\CodeSpecialException
     * @throws \App\Exception\custom\InterfaceException
     */
    public static function bindPhone($params){
        self::$openid = !empty($params['openid']) ? $params['openid'] : '';
        $phone = !empty($params['mobile']) ? $params['mobile'] : '';
        $wechatFrom = !empty($params['appid']) ? $params['appid'] : '';
        self::setWechatConfig($wechatFrom);
        if (!empty($phone)){
            $userInfo = self::delivererBindPhone($params);
        }else{
            $userInfo = self::pinBindPhone($params);
        }
        return $userInfo;
    }

    /**
     * 配送员绑定手机号
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws UserException
     * @throws \App\Exception\custom\CodeSpecialException
     * @throws \App\Exception\custom\InterfaceException
     */
    public static function delivererBindPhone($params){
        $mobile = !empty($params['mobile']) ? $params['mobile'] : '';
        $is_mobile =  ( new Validate)->isMobile($mobile);
        if (empty($is_mobile)) {
            throw new UserException(['cus' => 14]);
        }
        //短信验证是否有效
        $check = self::checkPhoneCode($params);
        if (!empty($check) && $check['status'] == 'failed') {
            throw new BaseException(['code' => $check['code'], 'msg' => $check['reason']]);
        }
        $jwUser = (new JwUser ())->getUserInfo(['mobile' => $mobile]);
        $userId = '';
        if (!empty($jwUser['data'])) {
            $data = $jwUser['data'];
            $userId = !empty($data['userId']) ? $data['userId'] : '';
        }
        $userInfo = UserWechatBind::getRecordMaster(['openid' => self::$openid]);
        CommonService::activateDeliverer(['user_id' => $userId,'phone' => $mobile]);
        $userInfo = array_merge($userInfo,self::saveWechatPhone($mobile,$userInfo)) ;
        return $userInfo;
    }

    /**
     * 验证手机验证码
     * @param array $params
     * @return array
     * @throws CodeSpecialException
     */
    public static function checkPhoneCode($params = [])
    {
        $url = config('interface','service.user.check_sms_code');
        if(!$url){
            throw new CodeSpecialException("failed");
        }
        $params = [
            'mobile' => !empty($params['mobile']) ? $params['mobile'] : '',
            'appid' => !empty($params['appid']) ? $params['appid'] : 'hubble',
            'code' => !empty($params['mobile_code']) ? $params['mobile_code'] : '',
            'key' => 'f16bdd46292480b4b82c841d90a6ba02',
            'partner' => 'merchant-b',
            'scene' => 'login',
        ];
        $sms  = (new Request())->get($url, $params);
        if(!$sms){
            throw new CodeSpecialException("timeout");
        }
        if (!empty($sms) && $sms['code'] == 0) {
            return $sms;
        }else{
            return $sms;
        }
    }

    /**
     * 拼一拼绑定手机号
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws UserException
     * @throws \App\Exception\custom\CodeSpecialException
     * @throws \App\Exception\custom\InterfaceException
     */
    public static function pinBindPhone($params)
    {
        $encryptedData = !empty($params['encryptedData']) ? $params['encryptedData'] : '';
        $iv = !empty($params['iv']) ? $params['iv'] : '';

        if (!$encryptedData) {
            FileLog::error("bindPhone：encryptedData 为空", json_encode($params, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 3]);
        }

        if (!$iv) {
            FileLog::error("bindPhone：iv 为空", json_encode($params, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 4]);
        }

        $userInfo = UserWechatBind::getRecordMaster(['openid' => self::$openid]);
        $sessionKey = !empty($userInfo['session_key']) ? $userInfo['session_key'] : '';
        $openid = isset($userInfo['openid']) && $userInfo['openid'] ? $userInfo['openid'] : '';
        if (empty($userInfo) || !$sessionKey || !$openid) {
            FileLog::error("bindPhone：userInfo获取失败", json_encode($userInfo, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 5]);
        }

        //解密
        $decryptData = [];
        $wXBizDataCrypt = new WxBizDataCrypt(self::$wechatAppId, $sessionKey);
        $errCode = $wXBizDataCrypt->decryptData($encryptedData, $iv, $decryptData);

        if ($errCode) {
            FileLog::error("bindPhone：解密失败", json_encode($params, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 6]);
        }

        // 手机号解密失败
        if (empty($decryptData['phoneNumber'])) {
            FileLog::error("bindPhone：手机号解密失败", json_encode($params, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            FileLog::error("bindPhone：手机号解密失败--decryptData", json_encode($decryptData, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 7]);
        }

        $phoneNumber = strval($decryptData['phoneNumber']);
        //判断是否已
        if ($params['appid'] == 'merchant-pub-pin') {
            $user = UserWechatBind::getRecordMaster(['phone' => $phoneNumber]);
        }else{
            $user = UserWechatBind::getRecordMaster(['phone' => $phoneNumber,'wechat_app_id' => self::$wechatAppId]);
        }
        
        if (!empty($user) &&  $openid != $user['openid']) {
            FileLog::error("bindPhone：该手机号已绑定其他用户不能重复绑定", json_encode($params, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            FileLog::error("bindPhone：该手机号已绑定其他用户不能重复绑定--user", json_encode($user, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 9]);
        }
        $userInfo = array_merge($userInfo,self::saveWechatPhone($phoneNumber,$userInfo)) ;
        return $userInfo;
    }

    public static function saveWechatPhone($phoneNumber,$userInfo){
        $jwUser = (new JwUser ())->getUserInfo(['mobile' => $phoneNumber]);
        $userId = '';
        if (!empty($jwUser['data'])) {
            $data = $jwUser['data'];
            $userId = !empty($data['userId']) ? $data['userId'] : '';
        }
        //进行绑定
        $update = [
            'phone' => $phoneNumber,
            "update_time" => date("Y-m-d H:i:s"),
            'user_id' => $userId,
        ];
        $bindStatus = UserWechatBind::save($update,['wechat_id' => $userInfo['wechat_id']]);

        if (empty($bindStatus)){
            FileLog::error("bindPhone：绑定手机号保存失败", json_encode($update, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 12]);
        }

        $isNewUser =  CommonService::isNewUser(['openid' => self::$openid]);
        if (!empty($isNewUser) && $isNewUser['is_new_user'] == 1) {
            $userInfo['is_pop_up'] = 1;
        }else{
            $userInfo['is_pop_up'] = 2;
        }

        $userInfo['user_id'] = $userId;
        $userInfo['phone'] = $phoneNumber;
        $userInfo['pop_up_text'] = "";
        return $userInfo;
    }

    /**
     * 绑定头像
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     */
    public static function userWechatBind($params)
    {

        $data['wechat_id[>]'] = 0;

        if (!empty($params['openid'])) {
            $data['openid'] = $params['openid'];
        }

        if (!empty($params['user_id'])) {
            $data['user_id'] = $params['user_id'];
        }

        if (!empty($params['phone'])) {
            $data['phone'] = $params['phone'];
        }

        if (!empty($params['wechat_app_id'])) {
            $data['wechat_app_id'] = $params['wechat_app_id'];
        }

        if (!empty($params['appid'])) {
            $data['appid'] = $params['appid'];
        }

        $userList = UserWechatBind::getRecords($data);
        return $userList;
    }


    public static function bindAvatar($params)
    {
        //进行绑定
        $update = [
            "update_time" => date("Y-m-d H:i:s"),
            'user_nick' => !empty($params['user_nick']) ? $params['user_nick'] : '',
            'user_avatar' =>!empty($params['user_avatar']) ? $params['user_avatar'] : '',
            'gender' => !empty($params['gender']) ? $params['gender'] : '',
            'country' => !empty($params['province']) ? $params['province'] : '',
            'province' => !empty($params['province']) ? $params['province'] : '',
            "city" => !empty($params['city']) ? $params['city'] : '',
            "language" => !empty($params['language']) ? $params['language'] : '',
        ];
        $user = UserWechatBind::getRecordMaster(['openid' => $params['openid']]);
        if (empty($user)){
            $update['openid'] = $params['openid'];
            $bindStatus = UserWechatBind::save($update);
        }else{
            $update['update_time'] = date("Y-m-d H:i:s");
            $bindStatus = UserWechatBind::save($update,['openid' => $params['openid']]);
        }
        if (!$bindStatus) {
            FileLog::error("bindAvatar：绑定头像失败", json_encode($params, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            FileLog::error("bindAvatar：绑定头像失败--update", json_encode($update, JSON_UNESCAPED_UNICODE),'','jianghaiming@yidian-inc.com');
            throw new UserException(['cus' => 11]);
        }
        return $update;
    }

    public static function generateUrllink($params)
    {
        //请求接口，获取用户openid
        $generateUrl = 'https://api.weixin.qq.com/wxa/generate_urllink?access_token='.self::getAccessToken();
        $marketingId = !empty($params['marketing_id']) ? $params['marketing_id'] : '';
        $params = [
            "path" => "/pages/product-info/index",
            "query"  => "marketing_id={$marketingId}&from=square",
            "is_expire"  => true,
            "expire_type"  => 1,
            "expire_interval"  => 10,
        ];
        $response = (new Request())->post($generateUrl, $params,5000,'json');
        $jsoninfo = $response['response'];
        return $jsoninfo;
    }

    /**
     * 获取accessToken
     * @return mixed
     */
    public static function getAccessToken()
    {
        $wechatFrom = 'merchant-pub-pin';
        $appid = \Yaf\Registry::get('config')->wechat->appid->$wechatFrom;
        $appsecret = \Yaf\Registry::get('config')->wechat->secret->$wechatFrom;
        $url = "https://api.weixin.qq.com/cgi-bin/token";
        $params = [
            'grant_type' => 'client_credential',
            'appid' => $appid,
            'secret' => $appsecret,
        ];
        $response = (new Request())->get($url, $params);
        $jsoninfo = $response['response'];
        $accessToken = $jsoninfo["access_token"];
        return $accessToken;
    }


    private static function generate3rdSession($openid) {
        return md5(mt_rand() . $openid);
    }

}